import { Injectable, NotFoundException } from "@nestjs/common";
import { DbService } from "apps/backend/src/app/db/db.service";
import {
	FilesForMachinesTable,
	MachinesTable,
} from "apps/backend/src/app/db/schema";
import { eq, ilike } from "drizzle-orm";

@Injectable()
export class MachinesService {
	constructor(private readonly database: DbService) {}

	/**
	 * Finds and returns a list of machines based on the given search parameters.
	 *
	 * @param {number} limit - The maximum number of machines to return.
	 * @param {number} offset - The number of machines to skip before starting to collect the result set.
	 * @param {string} searchField - The field to search within the machine names.
	 *
	 * @return {Promise<Array>} A promise that resolves to an array of found machines.
	 */
	async findMany(limit: number, offset: number, searchField: string) {
		console.log(`Searching machines. \nSearch : ${searchField}`);
		const machines = await this.database
			.use()
			.select()
			.from(MachinesTable)
			.where(ilike(MachinesTable.machineName, String(`%${searchField}%`)))
			.limit(limit)
			.offset(offset)
			.prepare("findMachineByName")
			.execute();
		console.log(`Found ${machines.length} machines.`);
		return machines;
	}

	//TODO The method to create a machine

	//TODO a method to delete a machine and delete the associated FilesTypeForMachine row

	/**
	 * Retrieves a list of files associated with a specific machine from the database.
	 *
	 * @param {number} limit - The maximum number of files to retrieve.
	 * @param {number} offset - The offset from which to start retrieving files.
	 * @param {string} searchField - The specific field to search within the files.
	 * @param {string} machineId - The unique identifier of the machine.
	 * @return {Promise<Array>} A promise that resolves to an array of file records for the machine.
	 * @throws {NotFoundException} If the machine with the given id is not found.
	 */
	async findFilesForMachine(
		limit: number,
		offset: number,
		searchField: string,
		machineId: string,
	) {
		const existingMachine = await this.database
			.use()
			.select()
			.from(MachinesTable)
			.where(eq(MachinesTable.id, machineId))
			.prepare("findMachineById")
			.execute();
		if (existingMachine.length !== 1)
			throw new NotFoundException("Machine id not found.");

		return await this.database
			.use()
			.select()
			.from(FilesForMachinesTable)
			.where(eq(FilesForMachinesTable.machineId, machineId))
			.limit(limit)
			.offset(offset)
			.prepare("findFilesForMachineId")
			.execute();
	}
}
